
#include "resource.h"
#include "os2.h"

/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
ICON            IDI_OS2EMBED    DISCARDABLE     "os2Embed.ICO"
ICON               IDI_SMALL    DISCARDABLE     "SMALL.ICO"

/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

MENU IDC_OS2EMBED DISCARDABLE 
BEGIN
    SUBMENU "~File", MOZ_File
    BEGIN
        MENUITEM "New Browser...",              MOZ_NewBrowser
        MENUITEM SEPARATOR
        MENUITEM "Switch Profile...",           MOZ_SwitchProfile
        MENUITEM SEPARATOR
        MENUITEM "~Save As...",                 MOZ_Save
        MENUITEM "Print Page",                  MOZ_Print
        MENUITEM "E~xit",                       IDM_EXIT
    END
    SUBMENU "~Edit", MOZ_Edit
    BEGIN
        MENUITEM "Cu~t",                        MOZ_Cut
        MENUITEM "~Copy",                       MOZ_Copy
        MENUITEM "~Paste",                      MOZ_Paste
        MENUITEM SEPARATOR
        MENUITEM "Select All",                  MOZ_SelectAll
        MENUITEM "Select None",                 MOZ_SelectNone
    END
    SUBMENU "~Go", MOZ_Go
    BEGIN
        MENUITEM "~Back",                       MOZ_GoBack
        MENUITEM "~Forward",                    MOZ_GoForward
    END
    SUBMENU "~Debug", MOZ_Debug
    BEGIN
        MENUITEM "~This space for rent",        ID_DEBUG_THISSPACEFORRENT
,         ,MIA_DISABLED
    END
    SUBMENU "~Help", MOZ_Help
    BEGIN
        MENUITEM "~About os2Embed...",          MOZ_About
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_APP_TITLE           "os2Embed"
    IDS_HELLO               "Embedding Mozilla is so much fun!!"
    IDS_ABOUT               "os2Embed - Gecko embedding sample"
    IDS_ABOUT_TITLE         "About os2Embed"
    IDS_HIST_BACK           "Going Back to: "
    IDS_HIST_FORWARD        "Going Forward to: "
    IDS_HIST_RELOAD_NORMAL  "Reloading url, (normal) :"
END

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_HIST_RELOAD_BYPASSPROXY "Reloading url, (bypassing Proxy) :"
    IDS_HIST_RELOAD_BYPASSCACHE "Reloading url, (bypassing cache) :"
    IDS_HIST_ADDURL         " added to Session History"
    IDS_HIST_RELOAD_BYPASSPROXYANDCACHE 
                            "Reloading url, (bypassing Proxy and cache) :"
    IDS_HIST_PURGE          "purged from session history"
    IDS_HIST_GOTO           "Going to history index : "
    IDS_HIST_URL            " URL : "
END


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

DLGTEMPLATE IDD_BROWSER DISCARDABLE  
BEGIN
  DIALOG "os2Embed sample - UNSUPPORTED", IDD_BROWSER, 0, 90, 500, 217,
  WS_VISIBLE | WS_CLIPSIBLINGS | NOT FS_DLGBORDER | FS_SIZEBORDER | FS_SCREENALIGN,
  FCF_TITLEBAR | FCF_SYSMENU | FCF_MINBUTTON | FCF_MAXBUTTON 
  BEGIN
    PUSHBUTTON    "Back",           IDC_BACK,   2, 193,  27,  13
    PUSHBUTTON    "Forward",     IDC_FORWARD,  29, 193,  41,  13
    PUSHBUTTON    "Reload",       IDC_RELOAD,  72, 193,  35,  13
    PUSHBUTTON    "Stop",           IDC_STOP, 108, 193,  32,  13             
    LTEXT         "Address:",     IDC_ADDRESSLABEL, 144, 196,  35,   8, SS_TEXT | DT_WORDBREAK | DT_MNEMONIC
    COMBOBOX      "",            IDC_ADDRESS, 182, 154, 242,  52, CBS_DROPDOWN | WS_TABSTOP
    DEFPUSHBUTTON "Go",               IDC_GO, 425, 193,  32,  13
    CONTROL       "Embedded Browser",IDC_BROWSER,0,  9, 500, 182, "OS2EMBED", WS_TABSTOP | WS_VISIBLE
    CONTROL       "Status",       IDC_STATUS,   0,   0, 395,   9, WC_STATIC, SS_TEXT | DT_LEFT | WS_GROUP | DT_MNEMONIC | WS_VISIBLE
    SLIDER                      IDC_PROGRESS, 395,   0, 105,   9, SLS_RIBBONSTRIP | SLS_READONLY | WS_VISIBLE
                                              CTLDATA 12, 0, 100, 0, 0, 0
  END
END

DLGTEMPLATE IDD_BROWSER_NC DISCARDABLE  
BEGIN
  DIALOG "os2Embed chromeless sample", IDD_BROWSER_NC, 0, 90, 500, 217,
  WS_VISIBLE | WS_CLIPCHILDREN | NOT FS_DLGBORDER | FS_SIZEBORDER | FS_SCREENALIGN,
  FCF_TITLEBAR | FCF_SYSMENU | FCF_MINBUTTON | FCF_MAXBUTTON | FCF_NOMOVEWITHOWNER
  BEGIN
    CONTROL       "Embedded Browser",IDC_BROWSER,0,  0, 500, 217, "OS2EMBED", WS_TABSTOP | WS_VISIBLE
  END
END


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

DLGTEMPLATE IDD_CHOOSEPROFILE DISCARDABLE  
BEGIN
  DIALOG "Choose Profile", IDD_CHOOSEPROFILE, 0, 182, 233, 154,
  WS_VISIBLE | FS_DLGBORDER | FS_SCREENALIGN,
  FCF_TITLEBAR | FCF_SYSMENU | FCF_NOMOVEWITHOWNER
  BEGIN
    LTEXT         "Available Profiles:",IDC_STATIC,9,139,78,   8, SS_TEXT | DT_WORDBREAK | DT_MNEMONIC
    LISTBOX                  IDC_PROFILELIST,   9,   7, 147, 129, LS_NOADJUSTPOS | WS_TABSTOP
    DEFPUSHBUTTON "Select",           DID_OK, 162, 122,  63,  14
    PUSHBUTTON    "Cancel",       DID_CANCEL, 162, 104,  63,  14
  END
END


