/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt (hewitt@netscape.com)
 *   Kevin Gerich (webmail@kmgerich.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== autocomplete.css =================================================
  == Styles used by the autocomplete widget.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
@namespace html url("http://www.w3.org/1999/xhtml");

/* ::::: autocomplete ::::: */

textbox {
  cursor: default;
  padding: 0;
}

/* Used by autocomplete widgets that don't have an icon. Gross. -dwh */
textbox.padded {
  padding: 1px 0px 1px 2px;
}

.textbox-input-box {
  margin: 0 3px;
  -moz-box-align: center;
}

/* ::::: history button ::::: */

.autocomplete-history-dropmarker {
  min-width: 17px;
  -moz-box-align: center;
  -moz-box-pack: center;
  padding: 0px;
  list-style-image: url("chrome://global/skin/icons/autocomplete-dropmarker.png");
  margin: 0px;
}

/* ::::: autocomplete popups ::::: */

popup[type="autocomplete"],
.autocomplete-history-popup {
  padding: 0px !important;
  background-color: -moz-Field !important;
  font: icon;
  -moz-appearance: none;
  border-top: 1px solid #C3C3C3 !important;
}

.autocomplete-history-popup {
  max-height: 180px;
  border-top: 1px solid #C3C3C3 !important;
}

/* ::::: tree ::::: */

.autocomplete-tree {
  -moz-appearance: none !important;
  border: none !important;
  background-color: transparent !important;
}

.autocomplete-treecol {
  -moz-appearance: none !important;
  margin: 0 !important;
  border: none !important;
  padding: 0 !important;
}

.autocomplete-treebody::-moz-tree-cell-text {
  padding-left: 2px;
}

treechildren.autocomplete-treebody::-moz-tree-row(selected) {
 background-color: Highlight;
}

treechildren.autocomplete-treebody::-moz-tree-cell-text(selected) {
  color: HighlightText !important;
}

/* ::::: textboxes inside toolbarpaletteitems ::::: */

toolbarpaletteitem > toolbaritem > textbox > hbox > hbox > html|*.textbox-input {
  visibility: hidden;
}

toolbarpaletteitem > toolbaritem > * > textbox > hbox > hbox > html|*.textbox-input {
  visibility: hidden;
}
